# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
"""


#-------------------------------------------------------------------------------
# Library odules import
#-------------------------------------------------------------------------------


import string
import Tix
from Tkconstants import *


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
import Base.XMLmodel as XMLmodel
from Pages.DefineUserScalars import DefineUserScalarsModel

#-------------------------------------------------------------------------------
# 
#-------------------------------------------------------------------------------


class ProbeValidator(TkPage.DataValidator2):
    """
    """
    def isProbeOK(self, entry, var):
        """
        This method validates the strict Positivity of the value, when an error
        occurs, an exception ValueError is raised.
        """
        try:
            if int(var) <= 0:
                return self._error(entry, ValueError, "value must be positive")
            else:
                return 1
        except:
            return self._error(entry, ValueError, "value must be positive")


#-------------------------------------------------------------------------------
# Output class
#-------------------------------------------------------------------------------


class VariableOption:
    """
    """
    def __init__(self, case, master, check, stbar, mdl, name, glob):
        """
         faire : afficher les variables en fonction du modele de turb
        limiter le nombre de caractre des entry  19
        test sur la cohrence des sondes (existence)
 
        on est oblig de passer la stbar en argument pour afficher les messages d'erreur:
                + de 19 caracteres pour le nom
                num des sondes existant et > 0
        """
        self.master = master
        self.case   = case
        self.check  = check
        self.stbar  = stbar
        self.mdl    = mdl
        self.name   = name
        self.glob   = glob
        self.list_turb_probe = self.mdl.getNodeProbeList()
        self.label_name      = self.mdl.dicoLabelName[name]
        self.color           = self.mdl.dicoColorName[name]
        self.balloon = Tix.Balloon(self.master, statusbar=self.stbar.label, bd=1, bg='black')
        self.balloon.message.config(bg=wm['background'])

        self.label    = Tix.StringVar()
        self.printing = Tix.StringVar()
        self.post     = Tix.StringVar()
        self.probe    = Tix.StringVar()

        self._createWidgets()


    def _createWidgets(self):
        """
        """
        t = PageText()
        f1 = Tix.Frame(self.master)
        f1.pack()

        self.lab = Tix.Entry(f1, width=11, textvariable=self.label, fg=self.color)
        self.lab.event_add("<<E>>", "<Return>", "<Leave>", "<FocusOut>")
        self.lab.bind("<<E>>", self.getLabel)
        
        self.balloon.bind_widget(self.lab, statusmsg=t.MSG_LABEL,
                                 balloonmsg=t.KEYWORD+"NOMVAR")

        butl = Tix.Checkbutton(f1, onvalue="on", offvalue="off",
                               variable=self.printing, command=self.getPrinting)
        self.balloon.bind_widget(butl, statusmsg=t.MSG_LISTING,
                                 balloonmsg=t.KEYWORD+"ILISVR")

        butp = Tix.Checkbutton(f1, onvalue="on", offvalue="off", 
                               variable=self.post, command=self.getPost)
        self.balloon.bind_widget(butp, statusmsg=t.MSG_POST,
                                 balloonmsg=t.KEYWORD+"ICHRVR")

        self.eProbes = Tix.Entry(f1, width=21, textvariable=self.probe)
        self.eProbes.event_add("<<Event2>>", "<Return>", "<Leave>", "<FocusOut>")
        self.eProbes.bind("<<Event2>>", self.getProbe)
        self.balloon.bind_widget(self.eProbes, statusmsg=t.MSG_PROBES,
                                 balloonmsg=t.KEYWORD+"IHISVR")

# popup for selection or none selection by probe

        pm_probe = Tix.PopupMenu(self.eProbes, title=t.POP_GLOBAL, state=NORMAL)
        pm_probe.bind_widget(self.eProbes)
        pm_probe.menu.add_separator()
        pm_probe.menu.add_command(label=t.SELECT_ALL, command=self.selectAll)
        pm_probe.menu.add_command(label=t.UNSELECT_ALL, command=self.cancelAll)
        pm_probe.menu.m1 = Tix.Menu(pm_probe.menu)

        self.lab.grid(row=0, column=0, pady=2)
        butl.grid(row=0, column=1, padx=28, pady=2)
        butp.grid(row=0, column=2, padx=18, pady=2)
        self.eProbes.grid(row=0, column=3, padx=2, pady=2)

        self.label.set(self.label_name)

##        if self.name in ('weight_matrix_X', 'weight_matrix_Y', 'weight_matrix_Z'):
##            self.printing.set('off')
##            self.mdl.setPrintingStatus(self.label_name, 'off')
##            self.post.set('off')
##            self.mdl.setPostStatus(self.label_name, 'off')
##            listProbe = []
##        else:
        self.printing.set(self.mdl.getPrintingStatus(self.label_name))
        self.post.set(self.mdl.getPostStatus(self.label_name))

        listProbe = []
##        listProbe = self.mdl.getNodeProbeList()
        listProbe = self.mdl.getProbesList(self.name)
        if listProbe: 
            self.probe.set(string.join(listProbe," "))

        if self.glob == 1: 
            self.selectAll()
            return
        if self.glob == 2:
            self.cancelAll()
            return


    def getLabel(self, event= None):
        t = PageText()

        self.check.begin(self.lab)
        label = self.label.get()
##        if label == self.label_nameself.mdl.getLabelNodeName(self.name): return
        if label == self.label_name: return

        self.stbar.busy()

        self.lab.unbind('<<E>>')

        if len(label) > LABEL_LENGTH_MAX:
            label = label[:LABEL_LENGTH_MAX]
        if label in self.mdl.getLabelsList():
            default = {}
            default['label'] = label
            default['list'] = self.mdl.getLabelsList()
            dialog = Dialog.VerifyExistenceLabel(self.master, default=default)
            select = dialog.result.copy()
            label = select['label']
            if select['label'] == default['label']:
                label = ''

        if len(label) == 0:
            self.check.error(self.lab)
            self.label.set(self.label_name)
        else:
            self.mdl.setNodeLabel(self.label_name, label)
            self.label_name = label
            self.label.set(label)
            
            self.mdl.updateBoundariesNodes(self.label_name, label)

        self.lab.event_add("<<E>>", "<Return>", "<Leave>", "<FocusOut>")
        self.lab.bind("<<E>>", self.getLabel)

        self.stbar.idle()


    def getPrinting(self, event= None):
        self.mdl.setPrintingStatus(self.label_name, self.printing.get() )
        return self.printing.get()


    def getPost(self, event= None):
        self.mdl.setPostStatus(self.label_name, self.post.get() )
        return self.post.get()

    def selectAll(self, event= None):
        self.probe.set(string.join(self.list_turb_probe))
        self.getProbe()


    def cancelAll(self, event= None):
        self.probe.set("")
        self.getProbe()


    def getProbe(self, event=None):
        t = PageText()
        self.stbar.clear()
        iok = 0
        ierr = []
        msg = ""
        self.check.begin(self.eProbes)
        for i in string.split(self.probe.get()):
            if not self.check.isProbeOK(self.eProbes,i): 
                iok = 1
            try:
                pr = int(i)
                if i not in self.list_turb_probe: ierr.append(i)
            except:
                pass
        if iok:
            msg= t.ERROR_INT
        if ierr:
            msg= msg + "  " + t.NOT_A_PROBE+ string.join(ierr)
        if iok or ierr:
            self.stbar.set(msg)
            self.check.error(self.eProbes)
        else:
            list = self.probe.get()
            self.mdl.updateProbesNodes(self.label_name, list)


#-------------------------------------------------------------------------------
# Model class
#-------------------------------------------------------------------------------

class OutputVariablesModel:

    def __init__(self, case):
        """
        Constuctor.
        """   
        self.case = case
        self.node_models    = self.case.xmlInitNode('thermophysical_models')
        self.analysis_ctrl  = self.case.xmlInitNode('analysis_control')
        self.fluid_prop     = self.case.xmlInitNode('physical_properties')
        self.node_model_vp  = self.node_models.xmlInitNode('velocity_pressure')
        self.node_var_vp    = self.node_model_vp.xmlGetNodeList('variable')
        self.node_pro_vp    = self.node_model_vp.xmlGetNodeList('property')
        self.node_ale       = self.node_models.xmlGetChildNode('ale_method')
        self.node_output    = self.analysis_ctrl.xmlInitNode('output')
        self.node_probe     = self.node_output.xmlGetNodeList('probe','name')
        self.node_means     = self.analysis_ctrl.xmlInitNode('time_averages')
        
        model = XMLmodel.XMLmodel(self.case)
        self.listNodeColor =   [(self.node_pro_vp,                   'black'),
                                (self.node_var_vp,                   'black'),
                                (model.getTurbNodeList(),            'black'),
                                (self.getThermalScalar(),            'brown'),
                                (self.getAdditionalScalar(),         'darkblue'),
                                (self.getAdditionalScalarProperty(), 'darkblue'),
                                (self.getFluidProperty(),            'darkgreen'),
                                (self.getTimeProperty(),             'darkgreen'),
                                (self.getPuCoalScalProper(),         'green'),
                                (self.getWeightMatrixProperty(),     'blue'),
                                (self.getListOfTimeMeans(),          'purple'),
                                (self.getListOfAleMethod(),          'orange')]
                                
        self.dicoLabelName = {}
        self.dicoColorName = {}
        self.list_name = []
        self.updateDicoLabelNameColor()


    def getThermalScalar(self):
        """
        Get node of thermal scalar
        """
        node_scalar = self.case.xmlGetNode('additional_scalars')
        thermal_scalar_list = node_scalar.xmlGetNodeList('scalar', type='thermal')
        return thermal_scalar_list


    def getPuCoalScalProper(self):
        nodList = []
        node = self.node_models.xmlGetNode('pulverized_coal', 'model')
        model = node['model']
        varList = []
        if model != 'off':
            nodList = []
            for var in ('scalar','property'):
                nodList = node.xmlGetNodeList(var)
                for nodvar in nodList: varList.append(nodvar)
        return varList


    def getAdditionalScalar(self):
        """
        """
        node0 = self.case.xmlGetNode('additional_scalars')
        return node0.xmlGetNodeList('scalar', type='user')


    def getAdditionalScalarProperty(self):
        """
        """
        nodeList = []
        for node in self.getAdditionalScalar():
            L = node.xmlGetNode('property',
                                choice='variable')
            if L: nodeList.append(L)

        return nodeList


    def getFluidProperty(self):
        """
        """
        nodeList = []
##        nodeThermal, model = self.getThermalModel()
        model = self.getThermalScalar()

        node = self.fluid_prop.xmlGetNode('fluid_properties')

        if node:
            for prop in ('density', 'molecular_viscosity', 'specific_heat'):
                L = node.xmlGetNode('property', name=prop, choice='variable')
                if L: nodeList.append(L)

            if model != '':
                L = node.xmlGetNode('property', name='thermal_conductivity', choice='variable')
                if L: nodeList.append(L)

        return nodeList


    def getWeightMatrixProperty(self):
        """
        """
        nodeList = []
        node0 = self.case.xmlGetNode('numerical_parameters')
        node1 = node0.xmlGetNode('velocity_pressure_coupling', 'status')
        if node1:
            if node1['status'] == 'on':
                nodeList = node0.xmlGetNodeList('property')
        return nodeList


    def getTimeProperty(self):
        """
        """
        nodeList = []

        node1 = self.analysis_ctrl.xmlGetNode('time_parameters')

        if node1:
            if node1.xmlGetInt('time_passing'):
                node2 = node1.xmlGetNode('property', name='local_time_step')
                if node2:
                    nodeList.append(node2)

            for prop in ('courant_number', 'fourier_number'):
                L = node1.xmlGetNode('property', name=prop)
                if L: nodeList.append(L)
        
        return nodeList
        
        
    def getListOfTimeMeans(self):
        """
        Get list of time averages variables
        """
        nodeList = []
        for node in self.node_means.xmlGetNodeList('time_average'):
            nodeList.append(node)
            
        return nodeList
        
        
        
    def getListOfAleMethod(self):
        """
        Get list of variables and properties for ale method if it's activated
        """
        nodeList = []
        if self.node_ale['status'] == 'on':
            for tag in ('variable', 'property'):
                for node in self.node_ale.xmlGetChildNodeList(tag):
                    nodeList.append(node)
                
        return nodeList


    def getLabelsList(self):
        """
        Get list of label for all variables, properties .....
        """
        model = XMLmodel.XMLmodel(self.case)
        list = []

        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                list.append(node['label'])
        return list


    def updateDicoLabelNameColor(self):
        """
        Get list of label for all variables, properties .....
        """
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                name = node['name']
                if not name: name = node['label']
                if not node['label']:
                    msg = "xml node named "+ name +" has no label"
                    raise ValueError, msg
                self.dicoLabelName[name] = node['label']
                self.dicoColorName[name] = color
                self.list_name.append(name)


    def getNodeProbeList(self):
        """
        Get list of node for probes
        """
        probeList = []
        for node in self.node_probe: 
            probeList.append(node['name'])
        return probeList


    def getProbesList(self, name):
        """
        Get list of probes if it exists for node['name'] = name
        """
        list = self.getNodeProbeList()
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['name'] == name:
                    node_probes = node.xmlGetChildNode('probes')
                    if node_probes:
                        nb_probes = node_probes['choice']
                        if nb_probes == '0':
                            list = []
                        elif nb_probes > '0':
                            list = []
                            for n in node_probes.xmlGetChildNodeList('probe_recording'):
                                list.append(n['name'])
        return list
                            
        
        
    def getPrintingStatus(self, label):
        """
        Get status for balise printing from node with label
        """
        status = 'on'

        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['label'] == label:
##                    name = node['name']
                    node_printing = node.xmlGetChildNode('listing_printing', 'status')
        if node_printing:
            status = node_printing['status']
##        else:
##            if name in ('weight_matrix_X', 'weight_matrix_Y', 'weight_matrix_Z'):
##                status = 'off'
##                self.setPrintingStatus(label, status)
        return status


    def getPostStatus(self, label):
        """
        Set status for balise post processing from node with label
        """
        status = 'on'
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['label'] == label:
                  node_post = node.xmlGetChildNode('postprocessing_recording', 'status')
        if node_post:
            status = node_post['status']
##        else:
##            if name in ('weight_matrix_X', 'weight_matrix_Y', 'weight_matrix_Z'):
##                status = 'off'
##                self.setPostStatus(label, status)
        return status


    def setNodeLabel(self, old_label, new_label):
        """
        Replace old_label by new_label for node with name and old_label
        """
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['label'] == old_label:
                    node['label'] = new_label

        self.updateDicoLabelNameColor()

##    def getLabelNodeName(self, name):
##        """
##        Get label of node with name name
##        """
##        for nodeList, color in self.listNodeColor:
##            for node in nodeList:
##                if node['name'] == name:
##                    label = node['label']
##        return label


    def updateBoundariesNodes(self, old_label, new_label):
        """
        Set good label for boundaries nodes with name and label
        """
        self.node_bc      = self.case.xmlInitNode('boundary_conditions')
        self.node_var     = self.node_bc.xmlInitNodeList('variable')
        self.node_sca     = self.node_bc.xmlInitNodeList('scalar')

        for node in [self.node_var, self.node_sca]:
            for nodebc in node:
                if nodebc['label'] == old_label:
                    nodebc['label'] = new_label


    def setPrintingStatus(self, label, status):
        """
        Set status for balise printing from node with name and label
        """
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['label'] == label:
                    if status == 'off':
                        node.xmlInitChildNode('listing_printing')['status'] = status
                    else:
                        if node.xmlGetChildNode('listing_printing'): 
                            node.xmlRemoveChild('listing_printing')


    def setPostStatus(self, label, status):
        """
        Set status for balise postprocessing from node with name and label
        """
        for nodeList, color in self.listNodeColor:
            for node in nodeList:
                if node['label'] == label:
                    if status == 'off':
                        node.xmlInitChildNode('postprocessing_recording')['status'] = status
                    else:
                        if node.xmlGetChildNode('postprocessing_recording'): 
                            node.xmlRemoveChild('postprocessing_recording')


    def updateProbesNodes(self, label, list):
        """
        Remove balise probe_recording if it exists
        """
        nb = len(string.split(list))
        if nb == len(self.getNodeProbeList()):
            return
        else:
            for nodeList, color in self.listNodeColor:
                for node in nodeList:
                    if node['label'] == label:
                        try:
                            node.xmlRemoveChild('probes')
                        except:
                            pass
                        n = node.xmlInitNode('probes', choice=str(nb))
                        if nb > 0: 
                            for i in string.split(list):
                                n.xmlInitChildNodeList('probe_recording',name=i)


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------


class OutputVariables(TkPage.Page):

    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['bounda/scalar']


    def _pageControl(self):
        """
        """
        self.check = ProbeValidator()
        self.info_turb_name = []
        self.mdl = OutputVariablesModel(self.case)


    def GlobSelectAll(self):
        """
        We select all probes for all variables as asked in popup
        """
        for name in self.mdl.dicoLabelName.keys():
                glob = 1
                VariableOption(self.case, self.f2.window,
                               self.check, self.stbar,
                               self.mdl, name, glob)


    def GlobCancelAll(self):
        """
        We select all probes for all variables as asked in popup
        """
        for name in self.mdl.dicoLabelName.keys():
                glob = 2
                VariableOption(self.case, self.f2.window,
                               self.check, self.stbar,
                               self.mdl, name, glob)


    def _createWidgets(self):
        """
        """
        t = PageText()

        lf1 = Tix.LabelFrame(self.myPage, bd=2, label=t.TITLE, relief=FLAT)
        lf1.label.config(font=fT)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        f1 = Tix.Frame(lf1.frame, relief=FLAT)
        f1.pack(side=TOP, padx=10, pady=10)

        self.l1 = Tix.Label(f1, text=t.NAME, bd=1, relief=GROOVE)
        self.l2 = Tix.Label(f1, text=t.LISTING, bd=1, relief=GROOVE)
        self.l3 = Tix.Label(f1, text=t.POST, bd=1, relief=GROOVE)
        self.l4 = Tix.Label(f1, text=t.PROBE, bd=1, relief=GROOVE)
        self.l1.grid (row=0, column=0, ipadx=26, ipady=8)
        self.l2.grid (row=0, column=1, ipadx=8, ipady=1)
        self.l3.grid (row=0, column=2, ipadx=8, ipady=1)
        self.l4.grid (row=0, column=3, ipadx=55, ipady=8)

        self.f2 = Tix.ScrolledWindow(lf1.frame, relief=FLAT)
        self.f2.pack()

# popup for global selection or none selection
        pm_global = Tix.PopupMenu(lf1.frame, title=t.POP_GLOBAL, state=NORMAL)
        pm_global.bind_widget(lf1.frame)
        pm_global.menu.add_separator()
        pm_global.menu.add_command(label=t.GLOB_SELECT_ALL, command=self.GlobSelectAll)
        pm_global.menu.add_command(label=t.GLOB_UNSELECT_ALL, command=self.GlobCancelAll)
        pm_global.menu.m1 = Tix.Menu(pm_global.menu)


    def _initializeWidgets(self):
        """
        """
        for name in self.mdl.list_name:
            glob = 0
            VariableOption(self.case, self.f2.window,
                           self.check, self.stbar,
                           self.mdl, name, glob)


#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.ERROR_8_CAR      = "Le nombre de caractres doit etre gal ou infrieur  19"
            self.ERROR_INT        = "Le numro de la sonde doit tre > 0"
            self.GLOB_SELECT_ALL  = "Selectionner toutes les sondes pour toutes les variables"
            self.GLOB_UNSELECT_ALL= "Dselectionner toutes les sondes pour toutes les variables"
            self.KEYWORD          = "Mot cl Code_Saturne: "
            self.LISTING          = "Affichage\nlisting"
            self.NAME             = "Nom"
            self.NOT_A_PROBE      = "Erreur, numro de sonde(s) inexistant(s) : "
            self.POP_GLOBAL       = "Menu popup"
            self.POST             = "Post-\nprocessing"
            self.PROBE            = "Sondes"
            self.SELECT_ALL       = "Tout slectionner"
            self.TITLE            = "Suivi des variables / scalaires"
            self.UNSELECT_ALL     = "Tout dslectionner"
        else:
            self.ERROR_8_CAR      = "Number of character must be lower or equal than 19"
            self.ERROR_INT        = "Probe Number must be an integer > 0"
            self.GLOB_SELECT_ALL  = "Select all probes for all variables"
            self.GLOB_UNSELECT_ALL= "Unselect all probes for all variables"
            self.KEYWORD          = "Code_Saturne key word: "
            self.LISTING          = "Print in\nlisting"
            self.NAME             = "Name"
            self.NOT_A_PROBE      = "Error, this probe doesn't exist: " 
            self.POP_GLOBAL       = "Menu popup"
            self.POST             = "Post-\nprocessing"
            self.PROBE            = "Probes"
            self.SELECT_ALL       = "Select all"
            self.TITLE            = "Control Solution"
            self.UNSELECT_ALL     = "Unselect all"
        # 2) Messages
        #

        if Tool.GuiParam.lang == 'fr':
            self.MSG_LABEL     = "Nom de la Variable/Scalaire max 32 caractres"
            self.MSG_LISTING   = "Suivi dans le listing"
            self.MSG_POST      = "Suivi en Post-processing"
            self.MSG_PROBES    = "Entrer le numero des sondes"
        else:
            self.MSG_LABEL     = "Variable/Scalar name =< 32 characters"
            self.MSG_LISTING   = "Printing listing"
            self.MSG_POST      = "Post-processing"
            self.MSG_PROBES    = "Enter Probes number"

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
