# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the scalars boundaries model data management.

This module contains the following classes and function:
- ScalarsBoundariesModel
- ScalarBC
- ScalarsBoundariesView
- PageText
- ScalarsBoundariesTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import sys, unittest
import Tix
from Tkconstants import *


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Pages.DefineBoundaryRegions import CreateHlistWidgets, MultiSelect
from Pages.DefineBoundaryRegions import DefBCModel


#-------------------------------------------------------------------------------
# Scalar Boundaries model class
#-------------------------------------------------------------------------------

class ScalarsBoundariesModel(TkPage.Page):
    """
    """
    def __init__(self, case):
        """
        Constructor
        """
        self.case = case
        self.radiative = self.case.xmlInitNode('radiative_transfer')
        self.thermal = self.case.xmlInitNode('thermal_scalar')
        self.node_add_scalar = self.case.xmlInitNode('additional_scalars')
        self.node_bc = self.case.xmlInitNode('boundary_conditions')
        self.node_def = self.node_bc.xmlInitNode('boundary_definition')

        t = PageText()

        # Definition of boundaries natures (same as BoundaryDef.py)
        #
        self.natureList = {'wall':   t.WALL   ,
                           'inlet':  t.INLET  ,
                           'outlet': t.OUTLET }


    def getThermalModel(self):
        from ThermalScalar import ThermalScalarModel
        model = ThermalScalarModel(self.case).getThermalScalar()
        if model != "off" :
            label = ThermalScalarModel(self.case).getThermalScalarLabel()
        else:
            label = ""
        del ThermalScalarModel

        return model, label


    def getThermalNode(self, nature, label):
        thermal, lab = self.getThermalModel()
        node = self.node_bc.xmlGetChildNode(nature, label=label)
        if node: node_th = node.xmlGetNode('scalar', type='thermal')
        return node_th


# TODO: a revoir avec Model de DefineUserScalar
    def addScalarList(self, nature, label):
        list = []
        node = self.node_bc.xmlGetChildNode(nature, label=label)
        if node: list = node.xmlGetNodeList('scalar', type='user')
        return list

 
    def getList(self, nature, label, str):
        list = []
        node = self.node_bc.xmlGetChildNode(nature, label=label)
        if node: list = node.xmlGetNodeList('scalar', type=str)
        return list


    def getNodeBound(self, label):
        return self.node_def.xmlGetNodeList(nature, label=label)


    def getBoundDefList(self, nature):
        return self.node_def.xmlGetNodeList(nature)


    #def mGetBoundCondList(self, nature, label):
    def getBoundCondList(self):
        return self.node_bc.xmlGetNodeList(nature, label)


    def getListBC(self, node, nature, label):
        return node.xmlGetNodeList(nature, label=label)


    def addWallChildBC(self, label):
        node = self.node_bc.xmlInitChildNode('wall', label=label)
        thermal, lab = self.getThermalModel()

        from DefineBoundaryRegions import DefBCModel
        DefBCModel(self.case).setCondScalar(node, 'neumann', 0.0)
        del DefBCModel


    def setDataNature(self, node, string, value):
        node.xmlSetData(string, value)


    def getStringNode(self, node, string):
        if not node.xmlGetString(string): 
            self.setDataNature(node, string, 0.0)
        return node.xmlGetString(string)


    def getChoice(self, node):
        return node['choice']


    def setChoice(self, node, string):
        node['choice'] = string


#-------------------------------------------------------------------------------
# ScalarBC class
#-------------------------------------------------------------------------------


class Scalar_BC:
    """
    """
    def __init__(self, case, master, check, stbar, node, natureFace, labelBR, color):
        """
        """
        self.master = master
        self.case   = case
        self.check = check
        self.stbar  = stbar
        self.natureFace = natureFace
        self.balloon = Tix.Balloon(self.master, 
                                   statusbar=self.stbar.label,
                                   bd=1, bg='black')
        self.balloon.message.config(bg=wm['background'])

        self.node = node
        self.labelBR = labelBR
        #self.nodeBR = self.node.xmlInitNodeList(self.natureFace,label=labelBR)[0]

        self.model = ScalarsBoundariesModel(self.case)

        t = PageText()

        self.dicoLabelName = {}
        
        self.label  = Tix.StringVar()
        self.natureCL = Tix.StringVar()
        self.value  = Tix.DoubleVar()
        self.coeff  = Tix.DoubleVar()

        self.label = self.node['label']

        fbc = Tix.Frame(self.master, relief=FLAT)
        fbc.pack(side=TOP)

        self.lab = Tix.Label(fbc, width=11, text=self.label, fg=color)
        self.lab.config(font = fL,  bd=2, relief=GROOVE)
        self.balloon.bind_widget(self.lab, statusmsg=t.LABEL_SCA,
                                 balloonmsg=t.KEYWORD+"NOMVAR")

        self.onature = Tix.OptionMenu(fbc, options='menubutton.width 45')
        self.onature.menubutton.config(bd=2, width=18, relief=RAISED)
        self.onature.add_command('dirichlet', label=t.DIRICHLET)
        self.onature.add_command('neumann', label=t.NEUMANN)
        self.onature.add_command('exchCoeff', label=t.COEFF)
        self.onature.config(variable=self.natureCL, command=self.getnatureCL)

# self.balloon.bind_widget(but, statusmsg=t.MSG_LISTING,
#                                 balloonmsg=t.KEYWORD+"ILISVR")

        self.eValue = Tix.Entry(fbc, width=8, textvariable=self.value)
        self.eValue.event_add("<<Event1>>", "<Return>", "<Leave>", "<FocusOut>")
        self.eValue.bind("<<Event1>>", self.getValue)
# self.balloon.bind_widget(self.eValue, statusmsg=t.VALUE,
#                                 balloonmsg=t.KEYWORD+"IHISVR")

        self.eCoeff = Tix.Entry(fbc, width=8, textvariable=self.coeff)
        self.eCoeff.event_add("<<Event2>>", "<Return>", "<Leave>", "<FocusOut>")
        self.eCoeff.bind("<<Event2>>", self.getCoeff)
# self.balloon.bind_widget(self.eCoeff, statusmsg=t.EXCH_COEFF,
#                                 balloonmsg=t.KEYWORD+"IHISVR")

        self.replace = Tix.Label(fbc, width=9)

        if self.natureFace == 'wall':
            self.lab.grid    (row=0, column=0, padx=4, pady=2, sticky=W)
            self.onature.grid   (row=0, column=1, padx=4, pady=2, sticky=W)
            self.eValue.grid (row=0, column=2, padx=4, pady=2, sticky=W)
        elif self.natureFace in [ 'inlet', 'outlet']:
            self.lab.grid    (row=0, column=0, padx=2, pady=2, sticky=W)
            self.onature.grid   (row=0, column=1, padx=2, pady=2, sticky=W)
            self.eValue.grid (row=0, column=2, padx=2, pady=2, sticky=W)
            if self.natureFace == 'inlet':
                self.onature.config(state=DISABLED)
            if self.natureFace == 'outlet':
                self.onature.disable('exchCoeff')
            self.replace.grid_forget()
        else:
            pass

# if self.nodeBR:
        if self.node:
        # Initialization 
            diriValue = self.model.getStringNode(self.node, 'dirichlet')
            if self.natureFace == 'wall':
                neumValue = self.model.getStringNode(self.node, 'neumann')
                exchValue = self.model.getStringNode(self.node, 'exchange_coefficient')
            if self.natureFace == 'outlet':
                neumValue = self.model.getStringNode(self.node, 'neumann')

            if self.natureFace == 'wall':
                self.nature = 'neumann'
            if self.natureFace == 'outlet':
                self.nature = 'neumann'
            else:
                self.nature = 'dirichlet'

            choice = self.model.getChoice(self.node)
            if choice == 'dirichlet':
                self.value.set(diriValue)
                self.nature = 'dirichlet'
                self.eCoeff.config(state=DISABLED)
                self.eCoeff.grid_forget()
                if self.natureFace == 'wall':
                    self.replace.grid(row=0, column=3, padx=4, pady=2, sticky=W)
                else:
                    self.replace.grid_forget()

            elif choice == 'neumann':
                self.value.set(neumValue)
                self.nature = 'neumann'
                if self.natureFace == 'wall':
                    self.eCoeff.grid_forget()
                    self.replace.grid(row=0, column=3, padx=4, pady=2, sticky=W)
                if self.natureFace == 'outlet':
                    self.onature.disable('exchCoeff')

            elif choice == 'exchange_coefficient':
                self.value.set(neumValue)
                if self.natureFace == 'wall':
                    self.replace.grid_forget()
                    self.eCoeff.grid (row=0, column=3, padx=4, pady=2, sticky=W)
                    self.coeff.set(exchValue)
                self.nature = 'exchCoeff'

        self.natureCL.set(self.nature)


    def getnatureCL(self, event=None):
        tt = self.natureCL.get()
        if tt == 'dirichlet': 
            self.model.setChoice(self.node, 'dirichlet')
            value = self.model.getStringNode(self.node, 'dirichlet')
            self.value.set(value)
            if self.natureFace == 'wall':
                self.eCoeff.grid_forget()
                self.replace.grid(row=0, column=3, padx=4, pady=2, sticky=W)

        elif tt == 'neumann': 
            self.model.setChoice(self.node, 'neumann')
            value = self.model.getStringNode(self.node, 'neumann')
            self.value.set(value)
            self.eCoeff.grid_forget()
            self.replace.grid(row=0, column=3, padx=4, pady=2, sticky=W)

        elif tt == 'exchCoeff': 
            self.replace.grid_forget()
            self.eCoeff.grid (row=0, column=3, padx=4, pady=2, sticky=W)
            self.model.setChoice(self.node, 'exchange_coefficient')
            value = self.model.getStringNode(self.node, 'neumann')
            self.value.set(value)
            coef = self.model.getStringNode(self.node, 'exchange_coefficient')
            self.coeff.set(coef)

 
    def getValue(self, event=None):
        self.stbar.busy()
        tt = self.natureCL.get()
        if self.check.hasType(self.eValue, self.value):
            value = self.value.get() 
            if not value: value = '0'
            if tt == 'dirichlet': self.model.setDataNature(self.node, 'dirichlet',value)
            if tt in ['neumann','exchCoeff']:
                self.model.setDataNature(self.node, 'neumann',value)
            self.stbar.idle()
            return self.value.get()
        self.stbar.idle()


    def getCoeff(self, event=None):
        self.stbar.busy()
        if self.check.hasType(self.eCoeff, self.coeff):
            coeff = self.coeff.get()
            if not coeff: coeff = '0'
            if self.natureCL.get() == 'exchCoeff':
                self.model.setDataNature(self.node, 'exchange_coefficient',coeff)
            self.stbar.idle()
            return self.coeff.get()
        self.stbar.idle()


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------


class ScalarsBoundariesView(TkPage.Page):
    """
    Class to open all Scalars (thermal, user and model) Boundary Conditions Page.
    Warning: turbulence variables (k, epsilon, Rij) are not considered
    like scalars.
    """
    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        pass


    def _pageControl(self):
        """
        Xml node declaration and supplementary default value settings.
        """
        t = PageText()

        self.model = ScalarsBoundariesModel(self.case)

        #  Initialize variables concerning the display of the Hlist1
        #
        self.mousePointedEntry = None
        self.selectedEntry = None
        self.currentEntry = None
        self.entriesNumber = 0


    def selectHList(self, entry=None):
        """
        Browsecmd:
        Every single Clik on the Button1 of the mouse send TWO signal to the GUI
        the first one when man push the button, and the second one when the
        button gets back.

        Command:
        Every double-Click and Return send only ONE signal to the GUI.
        """
        self.selectedEntry = self.mousePointedEntry
        self.mousePointedEntry = entry

        if (self.selectedEntry == self.mousePointedEntry
            and self.currentEntry != self.mousePointedEntry):
            if entry == None:
                msg = "There is an error in the use of the HList1.\n"\
                      "The selectionned entry is:" + entry + "\n" \
                      "The application will finish."
                tkMessageBox.showerror('SELECTED LIST ERROR', msg)
                sys.exit(EXIT_FAILURE)

            self.currentEntry = entry

            label, zone, nature, local = self.select.boundaryInfo(self.selectedEntry)

            for key in self.natureList.keys():
                if self.natureList[key] == nature:
                    nature = key
                    self.h.hlist.config(selectbackground=self.natureColor[key])

            self.s2.pack_forget()
            self.f3.pack_forget()
            self.f4.pack_forget()
            try:
                self.f31.destroy()
            except:
                pass
            try:
                self.f41.destroy()
            except:
                pass

            self._createWidgetsFac(nature, label)


    def addInHlist(self, label, zone, nature, local):
        """
        Create the item list associated with the new boundary definition.
        """
        self.stbar.busy()

        for key in self.model.natureList.keys():
            if self.natureList[key] == nature: nature = key

        self.entriesNumber = self.entriesNumber + 1
        name = 'entry' + repr(self.entriesNumber)

        self.h.hlist.add(name, itemtype=Tix.TEXT,
                          text=label,
                          style=self.style[nature+'b'])
        
        self.h.hlist.item_create(name, 1, itemtype=Tix.TEXT,
                                 text=zone,
                                 style=self.style[nature])

        self.h.hlist.item_create(name, 2, itemtype=Tix.TEXT,
                                  text=self.natureList[nature],
                                  style=self.style[nature])

        self.h.hlist.item_create(name, 3, itemtype=Tix.TEXT,
                                 text=local,
                                 style=self.style[nature])

        self.stbar.idle()


    def _createWidgetsFac(self, natureFac, label_BR):
        """
        Create the Page layout.
        """
        t = PageText()

        self.scalTh_BC = None
        self.scal_BC    = self.model.addScalarList(natureFac, label_BR)
        model, lab = self.model.getThermalModel()
        if model != 'off':
            self.scalTh_BC  = self.model.getThermalNode(natureFac, label_BR)
        self.scalMod_BC = self.model.getList(natureFac, label_BR,  'model')
    
        if self.scal_BC or self.scalTh_BC or self.scalMod_BC:
            self.s2.pack(side=TOP, fill=X, pady=10)
            if natureFac == 'wall':
                self.f3.pack(side=TOP, fill=X, pady=5)
                self.f31 = Tix.ScrolledWindow(self.lf.frame, relief=FLAT, height=190)
                self.f31.pack(side=TOP, fill=Y)

            elif natureFac in [ 'inlet', 'outlet']:
                self.f4.pack(side=TOP, fill=X, pady=5)
                self.f41 = Tix.ScrolledWindow(self.lf.frame, relief=FLAT, height=190)
                self.f41.pack(side=TOP, fill=Y)

            for scalar, color in [(self.scalTh_BC,  'brown'),
                                  (self.scalMod_BC, 'darkgreen'),
                                  (self.scal_BC,    'darkblue')]:
                if scalar == self.scalTh_BC:
                        node = scalar
                        if node:
                            if natureFac == 'wall':
                                w = self.f31.window
                            elif natureFac in [ 'inlet', 'outlet']:
                                w = self.f41.window
                            Scalar_BC(self.case, w, self.check2, self.stbar, 
                                      node, natureFac, label_BR, color)
                else:
                    for node in scalar:
                        if natureFac == 'wall':
                            w = self.f31.window
                        elif natureFac in [ 'inlet', 'outlet']:
                            w = self.f41.window
                        Scalar_BC(self.case, w, self.check2, self.stbar, 
                                  node, natureFac, label_BR, color)


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()

        self.lf = Tix.LabelFrame(self.myPage, bd=2, label=t.SCALARS_BOUNDARIES, relief=FLAT)
        self.lf.label.config(font=fT)
        self.lf.pack(side=TOP, fill=X, padx=6, pady=6)

        f0 = Tix.Frame(self.lf.frame, relief=SUNKEN)
        f0.pack(side=TOP, anchor=W)

        self.f1 = Tix.Frame(f0, relief=SUNKEN)
        self.f1.pack(side=LEFT, padx=10)

        self.s2 = Tix.Frame(self.lf.frame, height=2, bd=2, relief=SUNKEN)

        self.f3 = Tix.Frame(self.lf.frame, relief=FLAT)
        self.f4 = Tix.Frame(self.lf.frame, relief=FLAT)

        l1 = Tix.Label(self.f3, text=t.NAME, width=12, bd=1, relief=GROOVE)
        l2 = Tix.Label(self.f3, text=t.NATURE, width=22, bd=1, relief=GROOVE)
        l3 = Tix.Label(self.f3, text=t.VALUE, width=8, bd=1, relief=GROOVE)
        l4 = Tix.Label(self.f3, text=t.COEFF1, width=9, bd=1, relief=GROOVE)
        l1.grid (row=0, column=0, ipadx=8, ipady=9)
        l2.grid (row=0, column=1, ipadx=10, ipady=9)
        l3.grid (row=0, column=2, ipadx=8, ipady=9)
        l4.grid (row=0, column=3, ipadx=4, ipady=2)
    
        l1 = Tix.Label(self.f4, text=t.NAME, width=10, bd=1, relief=GROOVE)
        l2 = Tix.Label(self.f4, text=t.NATURE, width=10, bd=1, relief=GROOVE)
        l3 = Tix.Label(self.f4, text=t.VALUE, width=7, bd=1, relief=GROOVE)
        l1.grid (row=0, column=0, ipadx=15, ipady=9)
        l2.grid (row=0, column=1, ipadx=12, ipady=9)
        l3.grid (row=0, column=2, ipadx=10, ipady=9)

        hl = CreateHlistWidgets(self.lf.frame, self.balloon, 'browse', self.selectHList)
        self.h = hl.h
        self.style = hl.style
        self.natureList = hl.natureList
        self.natureColor = hl.natureColor

        # Create the mouse selection instance
        #
        self.select = MultiSelect(self.h.hlist, self.natureList, self.natureColor)


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        boundList = []
        dbr = DefBCModel(self.case)

        for nature in self.model.natureList.keys():
            for node in self.model.getBoundDefList(nature):
                label = node['label']
                zone = dbr.dicoLabel2Zone[label]
                local = dbr.getLocalisation(label, nature) 
                if (label,nature) not in boundList:
                    boundList.append((label,nature))
                    self.addInHlist(label, zone, nature, local)
                if nature == 'wall':
                    self.model.addWallChildBC(label)


#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.BOUNDARY_REGIONS    = "Zones de frontire"
            self.INLET               = "entre"
            self.OUTLET              = "sortie"
            self.LABEL               = "tiquette"
            self.LABEL_SCA           = "Nom du scalaire"
            self.NUM                 = "n"
            self.OFF                 = "non"
            self.ON                  = "oui"
            self.SCALARS             = "Scalaires"
            self.SCALARS_BOUNDARIES  = "Conditions aux limites des scalaires"
            self.SYMMETRY            = "symtrie"
            self.nature              = "nature"
            self.UNDEFINED           = "indfini"
            self.WALL                = "paroi"
            self.NAME                = "Nom du Scalaire"
            self.REF                 = "Rfrence(s)"
            self.NATURE              = "nature"
            self.VALUE               = "Valeur"
            self.COEFF1              = "Coefficient\nd'change"
            self.COEFF               = "Coefficient d'change"
            self.DIRICHLET           = "Valeur impose"
            self.NEUMANN             = "Flux impose"
            self.KEYWORD             = "Mot cl Code_Saturne: "
        else:
            self.BOUNDARY_REGIONS    = "Boundary regions"
            self.INLET               = "inlet"
            self.OUTLET              = "outlet"
            self.LABEL               = "Label"
            self.LABEL_SCA           = "Scalar name"
            self.NUM                 = " # "
            self.OFF                 = "off"
            self.ON                  = "on"
            self.SCALARS             = "Scalars"
            self.SCALARS_BOUNDARIES  = "Scalars boundary conditions"
            self.SYMMETRY            = "symmetry"
            self.nature              = "nature"
            self.UNDEFINED           = "undefined"
            self.WALL                = "wall"
            self.NAME                = "Scalar Name"
            self.REF                 = "Reference(s)"
            self.NATURE              = "nature"
            self.VALUE               = "Value"
            self.COEFF1              = "Exchange\nCoefficient"
            self.COEFF               = "Exchange coefficient"
            self.DIRICHLET           = "Imposed value"
            self.NEUMANN             = "Imposed flux"
            self.KEYWORD             = "Code_Saturne keyword: "

        # 2) Messages
        #
        if Tool.GuiParam.lang == 'fr':
            self.MSG_SYMMETRY   = "Aucune information\nutilisateur\nn'est ncessaire"
            self.MSG_UNDEFINED  = "Intervention utilisateur obligatoire\n"\
                                  "dans le sous-programe USCLIM"
        else:
            self.MSG_SYMMETRY   = "No user input is required"
            self.MSG_UNDEFINED  = "User-coding is required "\
                                  "in the subroutine USCLIM"


#-------------------------------------------------------------------------------
# TurbulenceModel test case
#-------------------------------------------------------------------------------


class ScalarBoundariesTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """
        This method is executed before all "check" methods.
        """
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        self.doc = XMLDocument()


    def tearDown(self):
        """
        This method is executed after all "check" methods.
        """
        del self.case
        del self.doc


    def checkScalarBoundariesInstantiation(self):
        """
        Check whether the ScalarBoundariesModel class could be instantiated
        """
        model = None
        model = ScalarsBoundariesModel(self.case)
        assert model != None, 'Could not instantiate ScalarBoundariesModel'


def suite():
    testSuite = unittest.makeSuite(ScalarBoundariesTestCase, "check")
    return testSuite

def runTest():
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
